/* vi:set cindent: */
#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <stdarg.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#define SMS_PAEGAS_CZ	"sms.paegas.cz"
#define HTTP_PORT	80
#define PAEGAS_URL	"/cgi-bin/sms2/wwwsms"
#define BUFSIZE 	10240

#define KEYPREFIX	"name=key value=\""
#define KEYPREFIXLEN	(sizeof(KEYPREFIX)-1)
#define MESSAGESENT	"Zprva odeslna"
#define MESSAGESENTLEN	(sizeof(MESSAGESENT)-1)
#define LIMIT		"Zkuste to za "
#define LIMITLEN	(sizeof(LIMIT)-1)
#define VALIDITY	"Platnost stranky"
#define VALIDITYLEN	(sizeof(VALIDITY)-1)
#define ENTER           "Zadejte daje"
#define ENTERLEN        (sizeof(ENTER)-1)


int debug;

int
sockprintf (int socket, char *format,...)
{
  va_list ap;
  char buf[10240];

  va_start (ap, format);
  vsprintf (buf, format, ap);
  va_end (ap);
  return write (socket, buf, strlen (buf));
}


int
smssend (char *number, char *email, unsigned char *message)
{
  char key[0x100];
  char url[8192];
  char *buf;
  unsigned char *msg;
  int len, nread;
  int i, j;
  int ntry;
  int delay;
  int goodhtml;
  char *num, *num2;
  int sock;
  unsigned long inaddr;
  struct sockaddr_in ad;
  struct hostent *hp;

  key[0] = 0;
  
  num = strdup(number);
  if (!num) {
      fprintf(stderr, "malloc: Not enough memory!\n");
      return -1;
  }

  /* 0042060Xxxxyyy */ 
  if (!strncmp(num,"0042060",7) && strlen(num) == 14)
      num2 = num + 7; else
  /* +42060Xxxxyyy */ 
  if (!strncmp(num,"+42060",6) && strlen(num) == 13)
      num2 = num + 6; else
  /* +420-60X-xxxyyy */ 
  if ((!strncmp(num,"+420-60",7)) && strlen(num) == 15 && (num[8] == '-')) {
      num[8] = num[7];
      num2 = num + 8;
  }else
  /* 060Xxxxyyy */
  if (!strncmp(num,"060",3) && strlen(num) == 10)
      num2 = num + 3; else
  /* 060X-xxxyyy */
  if (!strncmp(num,"060",3) && strlen(num) == 11 && (num[4] == '-')) {
      num[4] = num[3];
      num2 = num + 4;
  } else {
    fprintf(stderr,"Invalid phone number '%s'\n",num);
    return -1;
  }

  printf("Sending message to +420-60%c-%s... ",num2[0], num2+1);
  fflush(stdout);

  /* Resolve host */
  memset (&ad, 0, sizeof (ad));
  ad.sin_family = AF_INET;

  inaddr = inet_addr (SMS_PAEGAS_CZ);
  if (inaddr != INADDR_NONE)
    memcpy (&ad.sin_addr, &inaddr, sizeof (inaddr));
  else
    {
      hp = gethostbyname (SMS_PAEGAS_CZ);
      if (hp == NULL)
	{
	  perror ("gethostbyname");
	  return -1;
	}
      memcpy (&ad.sin_addr, hp->h_addr, hp->h_length);
    }
  ad.sin_port = htons (HTTP_PORT);

  ntry = 0;
  msg = (char *) malloc (10240);
  
  for (i=0,j=0;i<strlen(message);i++) {
      if (message[i]==' ')
	  msg[j++] = '+'; else
      if (message[i]<0x80)
          msg[j++] = message[i]; else
       {
         msg[j++]='%';
#define TOX(x)  ((x)>9?(x)+'A'-0xA:(x)+'0')
         msg[j++]=TOX(message[i]>>4);
         msg[j++]=TOX(message[i]&0xF);
#undef TOX
       }
  }
  msg[j] = 0;

      buf = (char *) malloc (10240);

  while (1)
    {
      ntry++;

      if (ntry > 10)
	  return -1;

      if (debug)
	  printf ("Connecting to %s...\n", inet_ntoa (ad.sin_addr));
      sock = socket (AF_INET, SOCK_STREAM, 0);
      if (sock < 0)
	{
	  perror ("socket");
	  return -1;
	}

      if (connect (sock, &ad, sizeof (ad)) < 0)
	{
	  perror ("connect");
	  return -1;
	}


      if (debug)
		printf("Connected, URL= %X\n",url);


      strcpy (url, PAEGAS_URL);

      if (debug) {
        printf("Number=%s\n", num2);
        printf("Email=%s\n", email);
        printf("Message=%s\n", msg);
        printf("Key=%s\n", key);
      }

      if (key[0])
	{
	  strcat (url, "?number=");
	  if (num2!=NULL)
               strcat (url, num2);
	  strcat (url, "&email=");
	  if (email!=NULL)
               strcat (url, email);
          strcat (url, "&mtype=1&text=");
          if (msg)
            strcat (url, msg);
	  strcat (url, "&cntr1=777&cntr2=0&");
	  strcat (url, "key=");
	  if (key!=NULL)
               strcat (url, key);
	}

	if (debug)
		printf("URL is (%X) %s\n", url, url);

      sockprintf (sock, "GET %s HTTP/1.0\r\n"
		  "Host: " SMS_PAEGAS_CZ "\r\n"
                  "User-Agent: Mozilla/4.0 (compatible; sms; Linux)\r\n"
		  "Referer: http://" SMS_PAEGAS_CZ PAEGAS_URL "\r\n\r\n",
		  url);

      if (debug)
	  printf ("Sending request...\n");
      len = 0;
      while ((nread = read (sock, buf + len, 10240 - len)) > 0) 
	  len += nread;
      close (sock);
      
      for (i = 0; i < len - KEYPREFIXLEN; i++)
	{
	  if (buf[i] == 'n')
	    if (!memcmp (buf + i, KEYPREFIX, KEYPREFIXLEN))
	      {
		i += KEYPREFIXLEN;
		for (j = i; j < len && buf[j] != '\"'; j++)
		  {
		    key[j - i] = buf[j];
		  }
		key[j - i] = 0;
	      }
	}

      if (key[0] == 0)
	{
	  printf ("Some error!\n");
	  return -1;
	}

      /* Check for success */
      for (i = 0; i < len - MESSAGESENTLEN; i++)
	  if (buf[i] == 'Z')
	    if (!memcmp (buf + i, MESSAGESENT, MESSAGESENTLEN))
	    {
		printf("Message sent successfully.\n");
		return 0;
	    }

      goodhtml = 0;
      /* Check for limit */
      for (i = 0; i < len - LIMITLEN; i++)
	  if (buf[i] == 'Z')
	    if (!memcmp (buf + i, LIMIT, LIMITLEN))
	    {
		i+=LIMITLEN;
		for (j=i;j<len;j++)
		    if (buf[j]==' ') {
			buf[j] = 0;
			delay = atoi(buf+i);
			goodhtml = 1;
		    }
	    }

      /* Check for validity */
      for (i = 0; i < len - VALIDITYLEN; i++)
	  if (buf[i] == 'P')
	    if (!memcmp (buf + i, VALIDITY, VALIDITYLEN))
	    {
		delay = 0;
		goodhtml = 1;
	    }

      /* Check for "Zadejte udaje" */
      for (i = 0; i < len - ENTERLEN; i++)
	  if (buf[i] == 'Z')
	    if (!memcmp (buf + i, ENTER, ENTERLEN))
	    {
		delay = 0;
		goodhtml = 1;
	    }


      if (!goodhtml) {
	  printf("\nWarning: bad HTML code, please contact the author.\n");
	  key[0] = 0;
      }
      
      //write (0, buf, len);
      
      
      if (ntry > 1 && delay){
	  struct timeval tv;

          if (debug)
	      printf("Sleeping %u seconds\n",delay);
	  else printf("[%u] ", delay);
	  fflush(stdout);

	  tv.tv_sec = delay;
	  tv.tv_usec = 0;
	  select(0, NULL, NULL, NULL, &tv);
      } else {
	  printf("* ");
	  fflush(stdout);
      }
      
    }
}

void usage(char *progname)
{
    printf("Usage: %s [-d] [-n email@address] [phone number(s)]\n"
	   "       %s -V\n"
	   "       %s -h\n"
	   "\n"
	   "Options:\n"
	   "    -h      Show this help screen\n"
	   "    -d      Debug mode\n"
	   "    -n      Send delivery notification via e-mail\n"
	   "    -V      Display version\n",
	   progname, progname, progname);
}

int main(int argc, char **argv)
{
    char *email;
    char *message;
    char line[0x100];
    char *phoneno;
    int havephone;
    char *progname;
    char ch;
    int errs;
    
    progname = argv[0];
    debug = 0;
    
    while ((ch = getopt(argc, argv, "Vhn:d"))!=-1) {
	switch(ch) {
	    case 'V':
		printf("Version 0.1 (compiled on " __DATE__ ")\n");
		break;
	    case 'h':
		usage(progname);
		exit(0);
 	    case 'n':
		email = optarg;
		break;
            case 'd':
		debug = 1;
		break;
	    default:
		fprintf(stderr,"Invalid option %c, try %s -h\n", ch, progname);
		exit(1);
	}
    }
    
    argc -= optind;
    argv += optind;
	
    havephone = argc!=0;
    
    message = (char *)malloc(1024);
   
    if (!havephone) {
      printf("Enter phone number: ");
      phoneno = (char *)malloc(1024);
      fgets(phoneno, 1024, stdin);
      if (phoneno[strlen(phoneno)-1] == '\n')
	  phoneno[strlen(phoneno)-1] = 0;
    }

    if (isatty(STDIN_FILENO)) 
	printf("Enter your message, end with '.':\n");

    message[0] = 0;    
    while (1) {
	if (isatty(STDIN_FILENO)) {
           printf("> ");
	   fflush(stdout);
        }
	if (fgets(line,0x100,stdin) == NULL)
	    break;
	if (line[strlen(line)-1] == '\n')
	    line[strlen(line)-1] = 0;
	if (!strcmp(line,"."))
	    break;
	if (message[0] && strlen(message)<1023)
	   strcat(message," ");
	if (strlen(message)+strlen(line)<1024)
		strcat(message,line);
	else {
	    memcpy(message, line, 1023-strlen(message));
	    message[1023] = 0;
	    printf("Warning: Message too long, truncated\n");
	    break;
	}
    }
 
    errs = 0;   
    if (havephone) {
	while(argc--) {
            if ( smssend(argv[0], email, message) < 0)
		errs++;
	    argv++;
	}
    }else{
	if ( smssend(phoneno, email, message) < 0)
	     errs++;
    }
    
    if (errs) {
	fprintf(stderr, "Error(s), some messages were not send!\n");
	exit(1);
    }else {
	printf("Ok, all messages sent to SMS center.\n");
	exit(0);
    }
    
    return 0;
}
